@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2018, Kovid Goyal <kovid at kovidgoyal.net>
 
@@ -208,7 +208,7 @@ class DiffHandler(Handler):
                 break
 
         if num is not None:
-            self.scroll_pos = min(num, self.max_scroll_pos)
+            self.scroll_pos = max(0, min(num, self.max_scroll_pos))
 
     @property
     def num_lines(self):
