@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>
 
@@ -275,7 +275,10 @@ def print_help_for_seq(seq, usage, message, appname):
     if print_help_for_seq.allow_pager and sys.stdout.isatty():
         import subprocess
         p = subprocess.Popen(['less', '-isRXF'], stdin=subprocess.PIPE)
-        p.communicate(text.encode('utf-8'))
+        try:
+            p.communicate(text.encode('utf-8'))
+        except KeyboardInterrupt:
+            raise SystemExit(1)
         raise SystemExit(p.wait())
     else:
         print(text)
@@ -474,7 +477,9 @@ Set the name part of the :italic:`WM_CLASS` property (defaults to using the valu
 
 --title -T
 Set the window title. This will override any title set by the program running inside kitty. So
-only use this if you are running a program that does not set titles.
+only use this if you are running a program that does not set titles. If combined
+with :option:`{appname} --session` the title will be used for all windows created by the
+session, that do not set their own titles.
 
 
 --config -c
@@ -546,9 +551,11 @@ specify this address. This option will be ignored, unless you set
 option as it is read automatically from the environment.
 
 
---start-in-fullscreen
-type=bool-set
-Make the initial kitty window fullscreen on startup
+--start-as
+type=choices
+default=normal
+choices=normal,fullscreen,maximized,minimized
+Control how the initial kitty window is created.
 
 
 # Debugging options
@@ -607,7 +614,9 @@ type=bool-set
 
 
 def options_for_completion():
-    return parse_option_spec(options_spec())[0]
+    raw = '--help -h\ntype=bool-set\nShow help for {appname} command line options\n\n{raw}'.format(
+            appname=appname, raw=options_spec())
+    return parse_option_spec(raw)[0]
 
 
 def option_spec_as_rst(ospec=options_spec, usage=None, message=None, appname=None, heading_char='-'):
@@ -701,7 +710,7 @@ def compare_opts(opts):
     compare_keymaps(final, initial)
 
 
-def create_opts(args, debug_config=False):
+def create_opts(args, debug_config=False, accumulate_bad_lines=None):
     from .config import load_config
     config = tuple(resolve_config(SYSTEM_CONF, defconf, args.config))
     if debug_config:
@@ -720,7 +729,7 @@ def create_opts(args, debug_config=False):
         if config:
             print(green('Loaded config files:'), ', '.join(config))
     overrides = (a.replace('=', ' ', 1) for a in args.override or ())
-    opts = load_config(*config, overrides=overrides)
+    opts = load_config(*config, overrides=overrides, accumulate_bad_lines=accumulate_bad_lines)
     if debug_config:
         compare_opts(opts)
     return opts
