@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
@@ -8,6 +8,7 @@ import re
 import sys
 from collections import namedtuple
 from contextlib import contextmanager
+from functools import partial
 
 from . import fast_data_types as defines
 from .conf.definition import as_conf_file, config_lines
@@ -16,13 +17,14 @@ from .conf.utils import (
     parse_config_base, python_string, to_bool, to_cmdline
 )
 from .config_data import all_options, parse_mods, type_map
-from .constants import cache_dir, defconf
+from .constants import cache_dir, defconf, is_macos
 from .utils import log_error
 
 named_keys = {
     "'": 'APOSTROPHE',
     ',': 'COMMA',
     '-': 'MINUS',
+    '_': 'UNDERSCORE',
     '.': 'PERIOD',
     '/': 'SLASH',
     ';': 'SEMICOLON',
@@ -69,7 +71,7 @@ def shlex_parse(func, rest):
 
 @func_with_args('combine')
 def combine_parse(func, rest):
-    sep, rest = rest.split(' ', 1)
+    sep, rest = rest.split(maxsplit=1)
     parts = re.split(r'\s*' + re.escape(sep) + r'\s*', rest)
     args = tuple(map(parse_key_action, filter(None, parts)))
     return func, args
@@ -77,7 +79,7 @@ def combine_parse(func, rest):
 
 @func_with_args('send_text')
 def send_text_parse(func, rest):
-    args = rest.split(' ', 1)
+    args = rest.split(maxsplit=1)
     if len(args) > 0:
         try:
             args[1] = parse_send_text_bytes(args[1])
@@ -90,9 +92,9 @@ def send_text_parse(func, rest):
 @func_with_args('run_kitten', 'run_simple_kitten', 'kitten')
 def kitten_parse(func, rest):
     if func == 'kitten':
-        args = rest.split(' ', 1)
+        args = rest.split(maxsplit=1)
     else:
-        args = rest.split(' ', 2)[1:]
+        args = rest.split(maxsplit=2)[1:]
         func = 'kitten'
     return func, args
 
@@ -115,7 +117,7 @@ def float_parse(func, rest):
 
 @func_with_args('change_font_size')
 def parse_change_font_size(func, rest):
-    vals = rest.split(' ', 1)
+    vals = rest.strip().split(maxsplit=1)
     if len(vals) != 2:
         log_error('Invalid change_font_size specification: {}, treating it as default'.format(rest))
         args = [True, None, 0]
@@ -125,13 +127,13 @@ def parse_change_font_size(func, rest):
         if amt[0] in '+-':
             args[1] = amt[0]
             amt = amt[1:]
-        args[2] = float(amt)
+        args[2] = float(amt.strip())
     return func, args
 
 
 @func_with_args('clear_terminal')
 def clear_terminal(func, rest):
-    vals = rest.split(' ', 1)
+    vals = rest.strip().split(maxsplit=1)
     if len(vals) != 2:
         log_error('clear_terminal needs two arguments, using defaults')
         args = ['reset', 'active']
@@ -140,6 +142,16 @@ def clear_terminal(func, rest):
     return func, args
 
 
+@func_with_args('copy_to_buffer')
+def copy_to_buffer(func, rest):
+    return func, [rest]
+
+
+@func_with_args('paste_from_buffer')
+def paste_from_buffer(func, rest):
+    return func, [rest]
+
+
 @func_with_args('neighboring_window')
 def neighboring_window(func, rest):
     rest = rest.lower()
@@ -150,6 +162,27 @@ def neighboring_window(func, rest):
     return func, [rest]
 
 
+@func_with_args('resize_window')
+def resize_window(func, rest):
+    vals = rest.strip().split(maxsplit=1)
+    if len(vals) > 2:
+        log_error('resize_window needs one or two arguments, using defaults')
+        args = ['wider', 1]
+    else:
+        quality = vals[0].lower()
+        if quality not in ('taller', 'shorter', 'wider', 'narrower'):
+            log_error('Invalid quality specification: {}'.format(quality))
+            quality = 'wider'
+        increment = 1
+        if len(vals) == 2:
+            try:
+                increment = int(vals[1])
+            except Exception:
+                log_error('Invalid increment specification: {}'.format(vals[1]))
+        args = [quality, increment]
+    return func, args
+
+
 @func_with_args('move_window')
 def move_window(func, rest):
     rest = rest.lower()
@@ -173,8 +206,32 @@ def pipe(func, rest):
     return func, rest
 
 
+@func_with_args('nth_window')
+def nth_window(func, rest):
+    try:
+        num = int(rest)
+    except Exception:
+        log_error('Invalid nth_window number: {}'.format(rest))
+        num = 1
+    return func, [num]
+
+
+@func_with_args('disable_ligatures_in')
+def disable_ligatures_in(func, rest):
+    parts = rest.split(maxsplit=1)
+    if len(parts) == 1:
+        where, strategy = 'active', parts[0]
+    else:
+        where, strategy = parts
+    if where not in ('active', 'all', 'tab'):
+        raise ValueError('{} is not a valid set of windows to disable ligatures in'.format(where))
+    if strategy not in ('never', 'always', 'cursor'):
+        raise ValueError('{} is not a valid disable ligatures strategy'.format(strategy))
+    return func, [where, strategy]
+
+
 def parse_key_action(action):
-    parts = action.split(' ', 1)
+    parts = action.strip().split(maxsplit=1)
     func = parts[0]
     if len(parts) == 1:
         return KeyAction(func, ())
@@ -185,7 +242,8 @@ def parse_key_action(action):
             func, args = parser(func, rest)
         except Exception:
             log_error('Ignoring invalid key action: {}'.format(action))
-    return KeyAction(func, args)
+        else:
+            return KeyAction(func, args)
 
 
 all_key_actions = set()
@@ -206,7 +264,10 @@ class KeyDefinition:
 
 
 def parse_key(val, key_definitions):
-    sc, action = val.partition(' ')[::2]
+    parts = val.split(maxsplit=1)
+    if len(parts) != 2:
+        return
+    sc, action = parts
     sc, action = sc.strip().strip(sequence_sep), action.strip()
     if not sc or not action:
         return
@@ -246,7 +307,7 @@ def parse_key(val, key_definitions):
 
 
 def parse_symbol_map(val):
-    parts = val.split(' ')
+    parts = val.split()
     symbol_map = {}
 
     def abort():
@@ -302,6 +363,15 @@ special_handlers = {}
 
 def special_handler(func):
     special_handlers[func.__name__.partition('_')[2]] = func
+    return func
+
+
+def deprecated_handler(*names):
+    def special_handler(func):
+        for name in names:
+            special_handlers[name] = func
+        return func
+    return special_handler
 
 
 @special_handler
@@ -326,6 +396,15 @@ def handle_clear_all_shortcuts(key, val, ans):
         ans['key_definitions'] = [None]
 
 
+@deprecated_handler('x11_hide_window_decorations', 'macos_hide_titlebar')
+def handle_deprecated_hide_window_decorations_aliases(key, val, ans):
+    if not hasattr(handle_deprecated_hide_window_decorations_aliases, key):
+        handle_deprecated_hide_window_decorations_aliases.key = True
+        log_error('The option {} is deprecated. Use hide_window_decorations instead.'.format(key))
+    if to_bool(val):
+        ans['hide_window_decorations'] = True
+
+
 def expandvars(val, env):
 
     def sub(m):
@@ -337,7 +416,7 @@ def expandvars(val, env):
             result = m.group()
         return result
 
-    return re.sub(r'$\{(\S+)\}', sub, val)
+    return re.sub(r'\$\{(\S+?)\}', sub, val)
 
 
 @special_handler
@@ -362,7 +441,7 @@ def option_names_for_completion():
     yield from special_handlers
 
 
-def parse_config(lines, check_keys=True):
+def parse_config(lines, check_keys=True, accumulate_bad_lines=None):
     ans = {'symbol_map': {}, 'keymap': {}, 'sequence_map': {}, 'key_definitions': [], 'env': {}}
     parse_config_base(
         lines,
@@ -370,7 +449,8 @@ def parse_config(lines, check_keys=True):
         type_map,
         special_handling,
         ans,
-        check_keys=check_keys
+        check_keys=check_keys,
+        accumulate_bad_lines=accumulate_bad_lines
     )
     return ans
 
@@ -468,13 +548,17 @@ def initial_window_size_func(opts, cached_values):
     w, w_unit = opts.initial_window_width
     h, h_unit = opts.initial_window_height
 
-    def get_window_size(cell_width, cell_height, dpi_x, dpi_y):
+    def get_window_size(cell_width, cell_height, dpi_x, dpi_y, xscale, yscale):
+        if not is_macos:
+            # scaling is not needed on Wayland, but is needed on macOS. Not
+            # sure about X11.
+            xscale = yscale = 1
         if w_unit == 'cells':
-            width = cell_width * w + (dpi_x / 72) * (opts.window_margin_width + opts.window_padding_width) + 1
+            width = cell_width * w / xscale + (dpi_x / 72) * (opts.window_margin_width + opts.window_padding_width) + 1
         else:
             width = w
         if h_unit == 'cells':
-            height = cell_height * h + (dpi_y / 72) * (opts.window_margin_width + opts.window_padding_width) + 1
+            height = cell_height * h / yscale + (dpi_y / 72) * (opts.window_margin_width + opts.window_padding_width) + 1
         else:
             height = h
         return width, height
@@ -536,10 +620,15 @@ def finalize_keys(opts):
     opts.sequence_map = sequence_map
 
 
-def load_config(*paths, overrides=None):
-    opts = _load_config(Options, defaults, parse_config, merge_configs, *paths, overrides=overrides)
+def load_config(*paths, overrides=None, accumulate_bad_lines=None):
+    parser = parse_config
+    if accumulate_bad_lines is not None:
+        parser = partial(parse_config, accumulate_bad_lines=accumulate_bad_lines)
+    opts = _load_config(Options, defaults, parser, merge_configs, *paths, overrides=overrides)
     finalize_keys(opts)
     if opts.background_opacity < 1.0 and opts.macos_titlebar_color:
         log_error('Cannot use both macos_titlebar_color and background_opacity')
         opts.macos_titlebar_color = 0
+    if (is_macos and getattr(opts, 'macos_hide_titlebar', False)) or (not is_macos and getattr(opts, 'x11_hide_window_decorations', False)):
+        opts.hide_window_decorations = True
     return opts
