@@ -1,11 +1,11 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
 import ctypes
 import sys
 from functools import partial
-from math import ceil, pi, cos, sqrt
+from math import ceil, pi, cos, floor
 
 from kitty.config import defaults
 from kitty.constants import is_macos
@@ -115,28 +115,30 @@ def add_dline(buf, cell_width, position, thickness, cell_height):
 
 
 def add_curl(buf, cell_width, position, thickness, cell_height):
-    xfactor = 2.0 * pi / cell_width
-    yfactor = max(thickness, 2)
+    max_x, max_y = cell_width - 1, cell_height - 1
+    xfactor = 2.0 * pi / max_x
+    half_height = max(thickness // 2, 1)
 
-    def clamp_y(y):
-        return max(0, min(int(y), cell_height - 1))
-
-    def clamp_x(x):
-        return max(0, min(int(x), cell_width - 1))
-
-    def add_intensity(x, y, distance):
+    def add_intensity(x, y, val):
+        y += position
+        y = min(y, max_y)
         idx = cell_width * y + x
-        buf[idx] = min(
-            255, buf[idx] + int(255 * (1 - distance))
-        )
+        buf[idx] = min(255, buf[idx] + val)
 
-    for x_exact in range(cell_width):
-        y_exact = yfactor * cos(x_exact * xfactor) + position
-        y = clamp_y(ceil(y_exact))
-        x_before, x_after = map(clamp_x, (x_exact - 1, x_exact + 1))
-        for x in {x_before, x_exact, x_after}:
-            dist = sqrt((x - x_exact)**2 + (y - y_exact)**2) / 2
-            add_intensity(x, y, dist)
+    # Ensure all space at bottom of cell is used
+    if position + half_height < max_y:
+        position += max_y - (position + half_height)
+    if position + half_height > max_y:
+        position -= position + half_height - max_y
+
+    # Use the Wu antialias algorithm to draw the curve
+    # cosine waves always have slope <= 1 so are never steep
+    for x in range(cell_width):
+        y = half_height * cos(x * xfactor)
+        y1, y2 = floor(y), ceil(y)
+        i1 = int(255 * abs(y - y1))
+        add_intensity(x, y1, 255 - i1)
+        add_intensity(x, y2, i1)
 
 
 def render_special(
@@ -147,7 +149,11 @@ def render_special(
     ans = CharTexture if missing else CharTexture()
 
     def dl(f, *a):
-        f(ans, cell_width, *a)
+        try:
+            f(ans, cell_width, *a)
+        except Exception as e:
+            log_error('Failed to render {} at cell_width={} and cell_height={} with error: {}'.format(
+                f.__name__, cell_width, cell_height, e))
 
     if underline:
         t = underline_thickness
